/****************************************************************************
 * crypto/manager.c
 *
 *   Copyright (C) 2018 Sebastien Lorquet. All rights reserved.
 *   Author:  Sebastien Lorquet <sebastien@lorquet.fr>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <nuttx/config.h>

#include <stdint.h>
#include <stdbool.h>

#include <errno.h>

#include <nuttx/kmalloc.h>
#include <nuttx/semaphore.h>
#include <nuttx/fs/fs.h>
#include <nuttx/crypto/module.h>
#include <nuttx/crypto/manager.h>

/****************************************************************************
 * Private Types
 ****************************************************************************/

struct cryptoman_session_s
{
  int current_module;                /* Used for module enumeration */
  FAR struct cryptomodule_s *module; /* Pointer to the current module */
  FAR void *session;                 /* Session handle within the module */
};

/****************************************************************************
 * Private Function Prototypes
 ****************************************************************************/

static int cryptoman_open (FAR struct file *filep);
static int cryptoman_close(FAR struct file *filep);
static int cryptoman_ioctl(FAR struct file *filep, int cmd,
                           unsigned long arg);

/****************************************************************************
 * Private Data
 ****************************************************************************/

static const struct file_operations cryptoman_fops =
{
  cryptoman_open,  /* open */
  cryptoman_close, /* close */
  NULL,            /* read */
  NULL,            /* write */
  NULL,            /* seek */
  cryptoman_ioctl  /* ioctl */
#ifndef CONFIG_DISABLE_POLL
  , NULL           /* poll */
#endif
#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
  , NULL           /* unlink */
#endif
};

static sem_t gUserSemaphore;

/****************************************************************************
 * Private Functions
 ****************************************************************************/

/****************************************************************************
 * Name: cryptoman_open
 * Description: Allocate a cryptographic management session to the
 *   user/task calling this function.
 ****************************************************************************/

static int cryptoman_open (FAR struct file *filep)
{
  FAR struct cryptoman_session_s *sess;

  sess = kmm_zalloc(sizeof(struct cryptoman_session_s));
  if(!sess)
    {
      return -ENOMEM;
    }

  filep->f_priv = sess;

  return 0;
}

/****************************************************************************
 * Name: cryptoman_open
 ****************************************************************************/

static int cryptoman_close(FAR struct file *filep)
{
  FAR struct cryptoman_session_s *sess = (FAR struct cryptoman_session_s*) filep->f_priv;

  if(sess)
    {
      /* Start by closing the module's session if a module has been selected */
      if(sess->module)
        {
        sess->module->ops->sessionfree(sess->session);
        }

      kmm_free(sess);
    }

  return 0;
}

/****************************************************************************
 * Name: cryptoman_modinfo
 ****************************************************************************/

static int cryptoman_modinfo(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_modinfo_s *modinfo)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_modselect
 ****************************************************************************/

static int cryptoman_modselect(FAR struct cryptoman_session_s *session,
                               FAR struct cryptoioc_modselect_s *modselect)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_modauth
 ****************************************************************************/

static int cryptoman_modauth(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_modauth_s *modauth)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keyinfo(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_keyinfo_s *keyinfo)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keycreate(FAR struct cryptoman_session_s *session,
                               FAR struct cryptoioc_keycreate_s *keycreate)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keydelete(FAR struct cryptoman_session_s *session,
                               FAR struct cryptoioc_keydelete_s *keydelete)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keygen(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_keygen_s *keygen)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keyupdate(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_keyupdate_s *keyupdate)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keyread(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_keyread_s *keyread)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_keyinfo
 ****************************************************************************/

static int cryptoman_keycopy(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_keycopy_s *keycopy)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_alginfo
 ****************************************************************************/

static int cryptoman_alginfo(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_alginfo_s *alginfo)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_alginfo
 ****************************************************************************/

static int cryptoman_alginit(FAR struct cryptoman_session_s *session,
                             FAR struct cryptoioc_alginit_s *alginit)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_alginfo
 ****************************************************************************/

static int cryptoman_algsetup(FAR struct cryptoman_session_s *session,
                              FAR struct cryptoioc_algsetup_s *algsetup)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_alginfo
 ****************************************************************************/

static int cryptoman_algupdate(FAR struct cryptoman_session_s *session,
                              FAR struct cryptoioc_algupdatefinish_s *algupdate)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_alginfo
 ****************************************************************************/

static int cryptoman_algfinish(FAR struct cryptoman_session_s *session,
                              FAR struct cryptoioc_algupdatefinish_s *algfinish)
{
  return -ENXIO;
};

/****************************************************************************
 * Name: cryptoman_ioctl
 ****************************************************************************/

static int cryptoman_ioctl(FAR struct file *filep, int cmd,
                           unsigned long arg)
{
  FAR struct cryptoman_session_s *sess = (FAR struct cryptoman_session_s*) filep->f_priv;

  switch(cmd)
    {
      case CRYPTOIOC_MODINFO    :
        return cryptoman_modinfo(sess, (FAR struct cryptoioc_modinfo_s*)arg);

      case CRYPTOIOC_MODSELECT  :
        return cryptoman_modselect(sess,
                                   (FAR struct cryptoioc_modselect_s*)arg);

      case CRYPTOIOC_MODAUTH    :
        return cryptoman_modauth(sess, (FAR struct cryptoioc_modauth_s*)arg);

      case CRYPTOIOC_KEYINFO    :
        return cryptoman_keyinfo(sess, (FAR struct cryptoioc_keyinfo_s*)arg);

      case CRYPTOIOC_KEYCREATE  :
        return cryptoman_keycreate(sess,
                                   (FAR struct cryptoioc_keycreate_s*)arg);

      case CRYPTOIOC_KEYDELETE  :
        return cryptoman_keydelete(sess,
                                   (FAR struct cryptoioc_keydelete_s*)arg);

      case CRYPTOIOC_KEYGENERATE:
        return cryptoman_keygen(sess, (FAR struct cryptoioc_keygen_s*)arg);

      case CRYPTOIOC_KEYUPDATE  :
        return cryptoman_keyupdate(sess,
                                   (FAR struct cryptoioc_keyupdate_s*)arg);

      case CRYPTOIOC_KEYREAD    :
        return cryptoman_keyread(sess, (FAR struct cryptoioc_keyread_s*)arg);

      case CRYPTOIOC_KEYCOPY    :
        return cryptoman_keycopy(sess, (FAR struct cryptoioc_keycopy_s*)arg);

      case CRYPTOIOC_ALGINIT    :
        return cryptoman_alginit(sess, (FAR struct cryptoioc_alginit_s*)arg);

      case CRYPTOIOC_ALGINFO    :
        return cryptoman_alginfo(sess, (FAR struct cryptoioc_alginfo_s*)arg);

      case CRYPTOIOC_ALGSETUP   :
        return cryptoman_algsetup(sess, (FAR struct cryptoioc_algsetup_s*)arg);

      case CRYPTOIOC_ALGUPDATE  :
        return cryptoman_algupdate(sess,
                                  (FAR struct cryptoioc_algupdatefinish_s*)arg);

      case CRYPTOIOC_ALGFINISH  :
        return cryptoman_algfinish(sess,
                                  (FAR struct cryptoioc_algupdatefinish_s*)arg);

    }
  return -ENOTTY;
}

/****************************************************************************
 * Public Functions
 ****************************************************************************/

/****************************************************************************
 * Name: devcrypto_register
 *
 * Description:
 *   Register the manager version of /dev/crypto
 *
 ****************************************************************************/

int devcrypto_register(void)
{
  nxsem_init(&gUserSemaphore, 0, 1);
  return register_driver("/dev/crypto", &cryptoman_fops, 0666, NULL);
}

/****************************************************************************
 * Name: cryptomod_register
 *
 * Description:
 *   Register a cryptographic module with the manager.
 *
 ****************************************************************************/

int cryptomod_register(FAR struct cryptomodule_s *module)
{
  return 0;
}

